#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from aodhclient import exceptions
from aodhclient.v2 import quota_cli


class QuotaShowTest(testtools.TestCase):
    def setUp(self):
        super().setUp()
        self.app = mock.Mock()
        self.quota_mgr_mock = self.app.client_manager.alarming.quota
        self.parser = mock.Mock()
        self.quota_show = (
            quota_cli.QuotaShow(self.app, self.parser))

    def test_quota_show(self):
        self.quota_mgr_mock.list.return_value = {
            "project_id": "fake_project",
            "quotas": [
                {
                    "limit": 20,
                    "resource": "alarms"
                }
            ]
        }
        parser = self.quota_show.get_parser('')
        args = parser.parse_args(['--project', 'fake_project'])
        # Something like [('alarms',), (20,)]
        ret = list(self.quota_show.take_action(args))

        self.quota_mgr_mock.list.assert_called_once_with(
            project='fake_project')
        self.assertIn('alarms', ret[0])
        self.assertIn(20, ret[1])


class QuotaSetTest(testtools.TestCase):
    def setUp(self):
        super().setUp()
        self.app = mock.Mock()
        self.quota_mgr_mock = self.app.client_manager.alarming.quota
        self.parser = mock.Mock()
        self.quota_set = (
            quota_cli.QuotaSet(self.app, self.parser))

    def test_quota_set(self):
        self.quota_mgr_mock.create.return_value = {
            "project_id": "fake_project",
            "quotas": [
                {
                    "limit": 20,
                    "resource": "alarms"
                }
            ]
        }

        parser = self.quota_set.get_parser('')
        args = parser.parse_args(['fake_project', '--alarm', '20'])
        ret = list(self.quota_set.take_action(args))

        self.quota_mgr_mock.create.assert_called_once_with(
            'fake_project', [{'resource': 'alarms', 'limit': 20}])
        self.assertIn('alarms', ret[0])
        self.assertIn(20, ret[1])

    def test_quota_set_invalid_quota(self):
        parser = self.quota_set.get_parser('')
        args = parser.parse_args(['fake_project', '--alarm', '-2'])

        self.assertRaises(exceptions.CommandError,
                          self.quota_set.take_action,
                          args)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cliff import lister

from aodhclient import utils

METRIC_COLS = ["name", "labels", "value"]


class CliMetrics(lister.Lister):
    """Get Metrics"""

    @staticmethod
    def metrics2cols(metrics):
        cols_data = []
        if "evaluation_results" in metrics:
            for alarm in metrics["evaluation_results"]:
                for state, count in alarm["state_counters"].items():
                    # prometheus style labels
                    labels = ('alarm_id="{{{}}}", project_id="{{{}}}", '
                              'state="{{{}}}"').format(alarm['alarm_id'],
                                                       alarm['project_id'],
                                                       state)
                    cols_data.append(["evaluation_result", labels, count])
        # Extend for other types of metrics here
        return METRIC_COLS, cols_data

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        metrics = utils.get_client(self).metrics.get()
        return self.metrics2cols(metrics)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import sys
from unittest import mock

from keystoneauth1 import exceptions
import testtools

from aodhclient import shell


class CliTest(testtools.TestCase):

    @mock.patch('sys.stderr', io.StringIO())
    def test_cli_http_error_with_details(self):
        shell.AodhShell().clean_up(
            None, None, exceptions.HttpError('foo', details='bar'))
        stderr_lines = sys.stderr.getvalue().splitlines()
        self.assertEqual(1, len(stderr_lines))
        self.assertEqual('bar', stderr_lines[0])

    @mock.patch('sys.stderr', io.StringIO())
    def test_cli_http_error_without_details(self):
        shell.AodhShell().clean_up(None, None, exceptions.HttpError('foo'))
        stderr_lines = sys.stderr.getvalue().splitlines()
        self.assertEqual(0, len(stderr_lines))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from aodhclient.tests.functional import base


class AlarmHistoryTest(base.ClientTestBase):

    def test_help(self):
        self.aodh("help", params="alarm-history show")
        self.aodh("help", params="alarm-history search")

    def test_alarm_history_scenario(self):

        PROJECT_ID = uuidutils.generate_uuid()
        RESOURCE_ID = uuidutils.generate_uuid()

        result = self.aodh('alarm',
                           params=("create "
                                   "--type gnocchi_resources_threshold "
                                   "--name history1 --metric cpu_util "
                                   "--threshold 5 "
                                   "--resource-id %s --resource-type generic "
                                   "--aggregation-method last "
                                   "--project-id %s"
                                   % (RESOURCE_ID, PROJECT_ID)))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        result = self.aodh('alarm',
                           params=("create "
                                   "--type gnocchi_resources_threshold "
                                   "--name history2 --metric cpu_util "
                                   "--threshold 10 "
                                   "--resource-id %s --resource-type generic "
                                   "--aggregation-method last "
                                   "--project-id %s"
                                   % (RESOURCE_ID, PROJECT_ID)))
        alarm = self.details_multiple(result)[0]
        ALARM_ID2 = alarm['alarm_id']

        # LIST WITH PAGINATION
        # list with limit
        result = self.aodh('alarm-history',
                           params=("show %s --limit 1" % ALARM_ID))
        alarm_list = self.parser.listing(result)
        self.assertEqual(1, len(alarm_list))
        # list with sort key=timestamp, dir=asc
        result = self.aodh('alarm-history',
                           params=("show %s --sort timestamp:asc" % ALARM_ID))
        alarm_history_list = self.parser.listing(result)
        timestamp = [r['timestamp'] for r in alarm_history_list]
        sorted_timestamp = sorted(timestamp)
        self.assertEqual(sorted_timestamp, timestamp)
        # list with sort key=type dir = desc and key=timestamp, dir=asc
        result = self.aodh('alarm-history',
                           params=("show %s --sort type:desc "
                                   "--sort timestamp:asc" % ALARM_ID))
        alarm_history_list = self.parser.listing(result)
        creation = alarm_history_list.pop(-1)
        timestamp = [r['timestamp'] for r in alarm_history_list]
        sorted_timestamp = sorted(timestamp)
        self.assertEqual(sorted_timestamp, timestamp)
        self.assertEqual('creation', creation['type'])

        # TEST FIELDS
        result = self.aodh(
            'alarm-history', params=("show %s" % ALARM_ID))
        history = self.parser.listing(result)[0]
        for key in ["timestamp", "type", "detail", "event_id"]:
            self.assertIn(key, history)

        # SHOW
        result = self.aodh(
            'alarm-history', params=("show %s" % ALARM_ID))
        history = self.parser.listing(result)[0]
        self.assertEqual('creation', history['type'])
        self.assertEqual('history1',
                         jsonutils.loads(history['detail'])['name'])

        result = self.aodh(
            'alarm-history', params=("show %s" % ALARM_ID2))
        history = self.parser.listing(result)[0]
        self.assertEqual('creation', history['type'])
        self.assertEqual('history2',
                         jsonutils.loads(history['detail'])['name'])

        # SEARCH ALL
        result = self.aodh('alarm-history', params=("search"))
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        self.assertIn(ALARM_ID2,
                      [r['alarm_id'] for r in self.parser.listing(result)])

        # SEARCH
        result = self.aodh('alarm-history',
                           params=("search --query "
                                   "alarm_id=%s"
                                   % ALARM_ID))
        history = self.parser.listing(result)[0]
        self.assertEqual(ALARM_ID, history["alarm_id"])
        self.assertEqual('creation', history['type'])
        self.assertEqual('history1',
                         jsonutils.loads(history['detail'])['name'])

        # CLEANUP
        self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.aodh('alarm', params="delete %s" % ALARM_ID2)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_utils import uuidutils
import requests
import requests.auth
from tempest.lib import exceptions

from aodhclient.tests.functional import base


class AodhClientTest(base.ClientTestBase):

    def test_help(self):
        self.aodh("help", params="alarm create")
        self.aodh("help", params="alarm delete")
        self.aodh("help", params="alarm list")
        self.aodh("help", params="alarm show")
        self.aodh("help", params="alarm update")
        self.aodh("help", params="alarm metrics")

    def test_alarm_id_or_name_scenario(self):
        def _test(name):
            params = "create --type event --name %s" % name
            result = self.aodh('alarm', params=params)
            alarm_id = self.details_multiple(result)[0]['alarm_id']

            params = 'show %s' % name
            result = self.aodh('alarm', params=params)
            self.assertEqual(alarm_id,
                             self.details_multiple(result)[0]['alarm_id'])

            params = 'show %s' % alarm_id
            result = self.aodh('alarm', params=params)
            self.assertEqual(alarm_id,
                             self.details_multiple(result)[0]['alarm_id'])

            params = "update --state ok %s" % name
            result = self.aodh('alarm', params=params)
            self.assertEqual("ok", self.details_multiple(result)[0]['state'])

            params = "update --state alarm %s" % alarm_id
            result = self.aodh('alarm', params=params)
            self.assertEqual("alarm",
                             self.details_multiple(result)[0]['state'])

            params = "update --name another-name %s" % name
            result = self.aodh('alarm', params=params)
            self.assertEqual("another-name",
                             self.details_multiple(result)[0]['name'])

            params = f"update --name {name} {alarm_id}"
            result = self.aodh('alarm', params=params)
            self.assertEqual(name,
                             self.details_multiple(result)[0]['name'])

            # Check update with no change is allowed
            params = f"update --name {name} {name}"
            result = self.aodh('alarm', params=params)
            self.assertEqual(name,
                             self.details_multiple(result)[0]['name'])

            params = "update --state ok"
            result = self.aodh('alarm', params=params,
                               fail_ok=True, merge_stderr=True)
            self.assertFirstLineStartsWith(
                result.splitlines(),
                'You need to specify one of alarm ID and alarm name(--name) '
                'to update an alarm.')

            params = "delete %s" % name
            result = self.aodh('alarm', params=params)
            self.assertEqual("", result)

            params = "create --type event --name %s" % name
            result = self.aodh('alarm', params=params)
            alarm_id = self.details_multiple(result)[0]['alarm_id']

            params = "delete %s" % alarm_id
            result = self.aodh('alarm', params=params)
            self.assertEqual("", result)

        _test(uuidutils.generate_uuid())
        _test('normal-alarm-name')

    def test_event_scenario(self):

        PROJECT_ID = uuidutils.generate_uuid()

        # CREATE
        result = self.aodh('alarm',
                           params=("create --type event --name ev_alarm1 "
                                   "--project-id %s" % PROJECT_ID))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('ev_alarm1', alarm['name'])
        self.assertEqual('*', alarm['event_type'])

        # UPDATE IGNORE INVALID
        result = self.aodh(
            'alarm', params=("update %s --severity critical --threshold 10"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])

        # UPDATE IGNORE INVALID
        result = self.aodh(
            'alarm', params=("update %s --event-type dummy" % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('dummy', alarm_updated['event_type'])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % ALARM_ID)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
        self.assertEqual('ev_alarm1', alarm_show['name'])
        self.assertEqual('dummy', alarm_show['event_type'])

        # GET BY NAME
        result = self.aodh(
            'alarm', params="show --name ev_alarm1")
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
        self.assertEqual('ev_alarm1', alarm_show['name'])
        self.assertEqual('dummy', alarm_show['event_type'])

        # GET BY NAME AND ID ERROR
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("show %s --name ev_alarm1" %
                                  ALARM_ID))

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
            if alarm_list["alarm_id"] == ALARM_ID:
                self.assertEqual('ev_alarm1', alarm_list['name'])

        # LIST WITH QUERY
        result = self.aodh('alarm',
                           params=("list --query project_id=%s" % PROJECT_ID))
        alarm_list = self.parser.listing(result)[0]
        self.assertEqual(ALARM_ID, alarm_list["alarm_id"])
        self.assertEqual('ev_alarm1', alarm_list['name'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # LIST DOES NOT HAVE ALARM
        result = self.aodh('alarm', params="list")
        self.assertNotIn(ALARM_ID,
                         [r['alarm_id'] for r in self.parser.listing(result)])

    def test_composite_scenario(self):

        project_id = uuidutils.generate_uuid()
        res_id = uuidutils.generate_uuid()
        # CREATE
        result = self.aodh(
            'alarm',
            params=('create --type composite --name calarm1 --composite-rule '
                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"},'
                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"},'
                    '{"threshold": 1000, "metric": "memory",'
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"}]}]}\' --project-id %s' %
                    (res_id, res_id, res_id, project_id)))
        alarm = self.details_multiple(result)[0]
        alarm_id = alarm['alarm_id']
        self.assertEqual('calarm1', alarm['name'])
        self.assertEqual('composite', alarm['type'])
        self.assertIn('composite_rule', alarm)

        # CREATE FAIL MISSING PARAM
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("create --type composite --name calarm1 "
                                  "--project-id %s" % project_id))

        # UPDATE
        result = self.aodh(
            'alarm', params=("update %s --severity critical" % alarm_id))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(alarm_id, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % alarm_id)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(alarm_id, alarm_show["alarm_id"])
        self.assertEqual(project_id, alarm_show["project_id"])
        self.assertEqual('calarm1', alarm_show['name'])

        # GET BY NAME
        result = self.aodh(
            'alarm', params="show --name calarm1")
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(alarm_id, alarm_show["alarm_id"])
        self.assertEqual(project_id, alarm_show["project_id"])
        self.assertEqual('calarm1', alarm_show['name'])

        # GET BY NAME AND ID ERROR
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("show %s --name calarm1" %
                                  alarm_id))

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(alarm_id,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
            if alarm_list["alarm_id"] == alarm_id:
                self.assertEqual('calarm1', alarm_list['name'])

        # LIST WITH QUERY
        result = self.aodh('alarm',
                           params=("list --query project_id=%s" % project_id))
        alarm_list = self.parser.listing(result)[0]
        self.assertEqual(alarm_id, alarm_list["alarm_id"])
        self.assertEqual('calarm1', alarm_list['name'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % alarm_id)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % alarm_id,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % alarm_id
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % alarm_id,
                           fail_ok=True, merge_stderr=True)
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # LIST DOES NOT HAVE ALARM
        result = self.aodh('alarm', params="list")
        self.assertNotIn(alarm_id,
                         [r['alarm_id'] for r in self.parser.listing(result)])

    def _test_alarm_create_show_query(self, create_params, expected_lines):

        def test(params):
            result = self.aodh('alarm', params=params)
            alarm = self.details_multiple(result)[0]
            for key, value in expected_lines.items():
                self.assertEqual(value, alarm[key])
            return alarm

        alarm = test(create_params)
        params = 'show %s' % alarm['alarm_id']
        test(params)
        self.aodh('alarm', params='delete %s' % alarm['alarm_id'])

    def test_event_alarm_create_show_query(self):
        params = ('create --type event --name alarm-multiple-query '
                  '--query "traits.project_id=789;traits.resource_id=012"')
        expected_lines = {
            'query': 'traits.project_id = 789 AND',
            '': 'traits.resource_id = 012',
        }
        self._test_alarm_create_show_query(params, expected_lines)

        params = ('create --type event --name alarm-single-query '
                  '--query "traits.project_id=789"')
        expected_lines = {'query': 'traits.project_id = 789'}
        self._test_alarm_create_show_query(params, expected_lines)

        params = 'create --type event --name alarm-no-query'
        self._test_alarm_create_show_query(params, {'query': ''})

    def test_set_get_alarm_state(self):
        result = self.aodh(
            'alarm',
            params=('create --type event --name alarm_state_test '
                    '--query "traits.project_id=789;traits.resource_id=012"'))
        alarm = self.details_multiple(result)[0]
        alarm_id = alarm['alarm_id']
        result = self.aodh(
            'alarm', params="show %s" % alarm_id)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual('insufficient data', alarm_show['state'])
        result = self.aodh('alarm', params="state get %s" % alarm_id)
        state_get = self.details_multiple(result)[0]
        self.assertEqual('insufficient data', state_get['state'])
        self.aodh('alarm',
                  params="state set --state ok  %s" % alarm_id)
        result = self.aodh('alarm', params="state get %s" % alarm_id)
        state_get = self.details_multiple(result)[0]
        self.assertEqual('ok', state_get['state'])
        self.aodh('alarm', params='delete %s' % alarm_id)

    def test_update_type_event_composite(self):

        res_id = uuidutils.generate_uuid()
        # CREATE
        result = self.aodh('alarm',
                           params=("create --type event --name ev_alarm123"))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('ev_alarm123', alarm['name'])
        self.assertEqual('*', alarm['event_type'])

        # UPDATE TYPE TO COMPOSITE
        result = self.aodh(
            'alarm',
            params=('update %s --type composite --composite-rule '
                    '\'{"or":[{"threshold": 0.8, "metric": "cpu_util", '
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"},'
                    '{"and": [{"threshold": 200, "metric": "disk.iops", '
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"},'
                    '{"threshold": 1000, "metric": "memory",'
                    '"type": "gnocchi_resources_threshold", "resource_type": '
                    '"generic", "resource_id": "%s", '
                    '"aggregation_method": "mean"}]}]}\'' %
                    (ALARM_ID, res_id, res_id, res_id)))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('composite', alarm_updated['type'])
        self.assertIn('composite_rule', alarm_updated)

        # UPDATE TYPE TO EVENT
        result = self.aodh(
            'alarm', params=("update %s --type event"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('event', alarm_updated['type'])
        self.assertEqual('*', alarm_updated['event_type'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)


class AodhClientGnocchiRulesTest(base.ClientTestBase):

    def test_gnocchi_resources_threshold_scenario(self):

        PROJECT_ID = uuidutils.generate_uuid()
        RESOURCE_ID = uuidutils.generate_uuid()

        req = requests.post(
            os.environ.get("GNOCCHI_ENDPOINT") + "/v1/resource/generic",
            headers={"X-Auth-Token": self.get_token()},
            json={
                "id": RESOURCE_ID,
            })
        self.assertEqual(201, req.status_code)

        # CREATE
        result = self.aodh('alarm',
                           params=("create "
                                   "--type gnocchi_resources_threshold "
                                   "--name alarm_gn1 --metric cpu_util "
                                   "--threshold 80 "
                                   "--resource-id %s --resource-type generic "
                                   "--aggregation-method last "
                                   "--project-id %s"
                                   % (RESOURCE_ID, PROJECT_ID)))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm_gn1', alarm['name'])
        self.assertEqual('cpu_util', alarm['metric'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertEqual('last', alarm['aggregation_method'])
        self.assertEqual(RESOURCE_ID,
                         alarm['resource_id'])
        self.assertEqual('generic', alarm['resource_type'])

        # CREATE WITH --TIME-CONSTRAINT
        result = self.aodh(
            'alarm',
            params=("create --type gnocchi_resources_threshold "
                    "--name alarm_tc --metric cpu_util --threshold 80 "
                    "--resource-id %s --resource-type generic "
                    "--aggregation-method last --project-id %s "
                    "--time-constraint "
                    "name=cons1;start='0 11 * * *';duration=300 "
                    "--time-constraint "
                    "name=cons2;start='0 23 * * *';duration=600 "
                    % (RESOURCE_ID, PROJECT_ID)))
        alarm = self.details_multiple(result)[0]
        self.assertEqual('alarm_tc', alarm['name'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertIsNotNone(alarm['time_constraints'])

        # CREATE FAIL MISSING PARAM
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("create "
                                  "--type gnocchi_resources_threshold "
                                  "--name alarm1 --metric cpu_util "
                                  "--resource-id %s --resource-type generic "
                                  "--aggregation-method last "
                                  "--project-id %s"
                                  % (RESOURCE_ID, PROJECT_ID)))

        # UPDATE
        result = self.aodh(
            'alarm', params=("update %s --severity critical --threshold 90"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])
        self.assertEqual('90.0', alarm_updated["threshold"])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % ALARM_ID)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
        self.assertEqual('alarm_gn1', alarm_show['name'])
        self.assertEqual('cpu_util', alarm_show['metric'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual('last', alarm_show['aggregation_method'])
        self.assertEqual('generic', alarm_show['resource_type'])

        # GET BY NAME
        result = self.aodh(
            'alarm', params="show --name alarm_gn1")
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
        self.assertEqual('alarm_gn1', alarm_show['name'])
        self.assertEqual('cpu_util', alarm_show['metric'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual('last', alarm_show['aggregation_method'])
        self.assertEqual('generic', alarm_show['resource_type'])

        # GET BY NAME AND ID ERROR
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("show %s --name alarm_gn1" %
                                  ALARM_ID))

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
            if alarm_list["alarm_id"] == ALARM_ID:
                self.assertEqual('alarm_gn1', alarm_list['name'])

        # LIST WITH PAGINATION
        # list with limit
        result = self.aodh('alarm',
                           params="list --filter all_projects=true --limit 1")
        alarm_list = self.parser.listing(result)
        self.assertEqual(1, len(alarm_list))
        # list with sort with key=name dir=asc
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --sort name:asc")
        names = [r['name'] for r in self.parser.listing(result)]
        sorted_name = sorted(names)
        self.assertEqual(sorted_name, names)
        # list with sort with key=name dir=asc and key=alarm_id dir=asc
        result = self.aodh(
            'alarm',
            params=("create --type gnocchi_resources_threshold "
                    "--name alarm_th --metric cpu_util --threshold 80 "
                    "--resource-id %s --resource-type generic "
                    "--aggregation-method last --project-id %s "
                    % (RESOURCE_ID, PROJECT_ID)))
        created_alarm_id = self.details_multiple(result)[0]['alarm_id']
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --sort name:asc "
                   "--sort alarm_id:asc")
        alarm_list = self.parser.listing(result)
        ids_with_same_name = []
        names = []
        for alarm in alarm_list:
            names.append(['alarm_name'])
            if alarm['name'] == 'alarm_th':
                ids_with_same_name.append(alarm['alarm_id'])
        sorted_ids = sorted(ids_with_same_name)
        sorted_names = sorted(names)
        self.assertEqual(sorted_names, names)
        self.assertEqual(sorted_ids, ids_with_same_name)
        # list with sort with key=name dir=desc and with the marker equal to
        # the alarm_id of the alarm_th we created for this test.
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --sort name:desc "
                   "--marker %s" % created_alarm_id)
        self.assertIn('alarm_tc',
                      [r['name'] for r in self.parser.listing(result)])
        self.aodh('alarm', params="delete %s" % created_alarm_id)

        # LIST WITH QUERY
        result = self.aodh('alarm',
                           params=("list --query project_id=%s" % PROJECT_ID))
        alarm_list = self.parser.listing(result)[0]
        self.assertEqual(ALARM_ID, alarm_list["alarm_id"])
        self.assertEqual('alarm_gn1', alarm_list['name'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # LIST DOES NOT HAVE ALARM
        result = self.aodh('alarm', params="list")
        self.assertNotIn(ALARM_ID,
                         [r['alarm_id'] for r in self.parser.listing(result)])

    def test_gnocchi_aggr_by_resources_scenario(self):

        # CREATE
        result = self.aodh(
            'alarm',
            params=("create "
                    "--type "
                    "gnocchi_aggregation_by_resources_threshold "
                    "--name alarm1 --metric cpu --threshold 80 "
                    "--query "
                    '\'{"=": {"creator": "cr3at0r"}}\' '
                    "--resource-type generic "
                    "--aggregation-method mean "))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm1', alarm['name'])
        self.assertEqual('cpu', alarm['metric'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertEqual('mean', alarm['aggregation_method'])
        self.assertEqual('generic', alarm['resource_type'])
        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                         alarm['query'])

        # CREATE FAIL MISSING PARAM
        self.assertRaises(
            exceptions.CommandFailed,
            self.aodh, 'alarm',
            params=("create "
                    "--type "
                    "gnocchi_aggregation_by_resources_threshold "
                    "--name alarm1 --metric cpu "
                    "--query "
                    '\'{"=": {"creator": "cr3at0r"}}\' '
                    "--resource-type generic "
                    "--aggregation-method mean "))

        # UPDATE
        result = self.aodh(
            'alarm', params=("update %s --severity critical --threshold 90"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])
        self.assertEqual('90.0', alarm_updated["threshold"])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % ALARM_ID)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual('alarm1', alarm_show['name'])
        self.assertEqual('cpu', alarm_show['metric'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual('mean', alarm_show['aggregation_method'])
        self.assertEqual('generic', alarm_show['resource_type'])

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
            if alarm_list["alarm_id"] == ALARM_ID:
                self.assertEqual('alarm1', alarm_list['name'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # LIST DOES NOT HAVE ALARM
        result = self.aodh('alarm', params="list")
        self.assertNotIn(ALARM_ID,
                         [r['alarm_id'] for r in self.parser.listing(result)])

    def test_gnocchi_aggr_by_metrics_scenario(self):

        PROJECT_ID = uuidutils.generate_uuid()
        METRIC1 = 'cpu'
        METRIC2 = 'cpu_util'

        # CREATE
        result = self.aodh(
            'alarm',
            params=("create "
                    "--type gnocchi_aggregation_by_metrics_threshold "
                    "--name alarm1 "
                    "--metrics %s "
                    "--metric %s "
                    "--threshold 80 "
                    "--aggregation-method last "
                    "--project-id %s"
                    % (METRIC1, METRIC2, PROJECT_ID)))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm1', alarm['name'])
        metrics = "['cpu', 'cpu_util']"
        self.assertEqual(metrics, alarm['metrics'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertEqual('last', alarm['aggregation_method'])

        # CREATE FAIL MISSING PARAM
        self.assertRaises(
            exceptions.CommandFailed,
            self.aodh, 'alarm',
            params=("create "
                    "--type gnocchi_aggregation_by_metrics_threshold "
                    "--name alarm1 "
                    "--metrics %s "
                    "--metrics %s "
                    "--aggregation-method last "
                    "--project-id %s"
                    % (METRIC1, METRIC2, PROJECT_ID)))

        # UPDATE
        result = self.aodh(
            'alarm', params=("update %s --severity critical --threshold 90"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])
        self.assertEqual('90.0', alarm_updated["threshold"])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % ALARM_ID)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual(PROJECT_ID, alarm_show["project_id"])
        self.assertEqual('alarm1', alarm_show['name'])
        self.assertEqual(metrics, alarm_show['metrics'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual('last', alarm_show['aggregation_method'])

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        for alarm_list in self.parser.listing(result):
            if alarm_list["alarm_id"] == ALARM_ID:
                self.assertEqual('alarm1', alarm_list['name'])

        # LIST WITH QUERY
        result = self.aodh('alarm',
                           params=("list --query project_id=%s" % PROJECT_ID))
        alarm_list = self.parser.listing(result)[0]
        self.assertEqual(ALARM_ID, alarm_list["alarm_id"])
        self.assertEqual('alarm1', alarm_list['name'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)

        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # LIST DOES NOT HAVE ALARM
        result = self.aodh('alarm', params="list")
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
        self.assertNotIn(ALARM_ID,
                         [r['alarm_id'] for r in self.parser.listing(result)])

    def test_update_gnresthr_gnaggrresthr(self):

        RESOURCE_ID = uuidutils.generate_uuid()
        # CREATE
        result = self.aodh('alarm',
                           params=("create "
                                   "--type gnocchi_resources_threshold "
                                   "--name alarm_gn123 --metric cpu_util "
                                   "--resource-id %s --threshold 80 "
                                   "--resource-type generic "
                                   "--aggregation-method last "
                                   % RESOURCE_ID))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm_gn123', alarm['name'])
        self.assertEqual('cpu_util', alarm['metric'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertEqual('last', alarm['aggregation_method'])
        self.assertEqual('generic', alarm['resource_type'])

        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
        result = self.aodh(
            'alarm', params=("update %s --type "
                             "gnocchi_aggregation_by_resources_threshold "
                             "--metric cpu --threshold 90 "
                             "--query "
                             '\'{"=": {"creator": "cr3at0r"}}\' '
                             "--resource-type generic "
                             "--aggregation-method mean "
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('cpu', alarm_updated['metric'])
        self.assertEqual('90.0', alarm_updated['threshold'])
        self.assertEqual('mean', alarm_updated['aggregation_method'])
        self.assertEqual('generic', alarm_updated['resource_type'])
        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                         alarm_updated['query'])
        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
                         alarm_updated['type'])

        # UPDATE TYPE TO GNOCCHI_RESOURCES_THRESHOLD
        result = self.aodh(
            'alarm', params=("update %s "
                             "--type gnocchi_resources_threshold "
                             "--metric cpu_util "
                             "--resource-id %s --threshold 80 "
                             "--resource-type generic "
                             "--aggregation-method last "
                             % (ALARM_ID, RESOURCE_ID)))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('cpu_util', alarm_updated['metric'])
        self.assertEqual('80.0', alarm_updated['threshold'])
        self.assertEqual('last', alarm_updated['aggregation_method'])
        self.assertEqual('generic', alarm_updated['resource_type'])
        self.assertEqual('gnocchi_resources_threshold',
                         alarm_updated['type'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

    def test_update_gnaggrresthr_gnaggrmetricthr(self):

        METRIC1 = 'cpu'
        METRIC2 = 'cpu_util'

        # CREATE
        result = self.aodh(
            'alarm',
            params=("create "
                    "--type "
                    "gnocchi_aggregation_by_resources_threshold "
                    "--name alarm123 --metric cpu --threshold 80 "
                    "--query "
                    '\'{"=": {"creator": "cr3at0r"}}\' '
                    "--resource-type generic "
                    "--aggregation-method mean "))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm123', alarm['name'])
        self.assertEqual('cpu', alarm['metric'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertEqual('mean', alarm['aggregation_method'])
        self.assertEqual('generic', alarm['resource_type'])
        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                         alarm['query'])

        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_METRICS_THRESHOLD
        result = self.aodh(
            'alarm', params=("update %s --type "
                             "gnocchi_aggregation_by_metrics_threshold "
                             "--metrics %s "
                             "--metrics %s "
                             "--threshold 80 "
                             "--aggregation-method last"
                             % (ALARM_ID, METRIC1, METRIC2)))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        metrics = "['cpu', 'cpu_util']"
        self.assertEqual(metrics, alarm_updated['metrics'])
        self.assertEqual('80.0', alarm_updated['threshold'])
        self.assertEqual('last', alarm_updated['aggregation_method'])
        self.assertEqual('gnocchi_aggregation_by_metrics_threshold',
                         alarm_updated['type'])

        # UPDATE TYPE TO GNOCCHI_AGGREGATION_BY_RESOURCES_THRESHOLD
        result = self.aodh(
            'alarm', params=("update %s --type "
                             "gnocchi_aggregation_by_resources_threshold "
                             "--metric cpu --threshold 80 "
                             "--query "
                             '\'{"=": {"creator": "cr3at0r"}}\' '
                             "--resource-type generic "
                             "--aggregation-method mean "
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('cpu', alarm_updated['metric'])
        self.assertEqual('80.0', alarm_updated['threshold'])
        self.assertEqual('mean', alarm_updated['aggregation_method'])
        self.assertEqual('generic', alarm_updated['resource_type'])
        self.assertEqual('{"=": {"creator": "cr3at0r"}}',
                         alarm_updated['query'])
        self.assertEqual('gnocchi_aggregation_by_resources_threshold',
                         alarm_updated['type'])

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)


class AodhClientPrometheusRulesTest(base.ClientTestBase):

    def test_prometheus_type_scenario(self):

        QUERY = "ceilometer_image_size"

        req = requests.get(
            os.environ.get("PROMETHEUS_ENDPOINT") +
            "/api/v1/status/runtimeinfo",
        )
        self.assertEqual(200, req.status_code)

        # CREATE
        result = self.aodh('alarm',
                           params=("create "
                                   "--type prometheus "
                                   "--name alarm_p1 "
                                   "--threshold 80 "
                                   "--query %s "
                                   % QUERY))
        alarm = self.details_multiple(result)[0]
        ALARM_ID = alarm['alarm_id']
        self.assertEqual('alarm_p1', alarm['name'])
        self.assertEqual(QUERY, alarm['query'])
        self.assertEqual('80.0', alarm['threshold'])

        # CREATE WITH --TIME-CONSTRAINT
        result = self.aodh(
            'alarm',
            params=("create --type prometheus "
                    "--name alarm_ptc --threshold 80 "
                    "--time-constraint "
                    "name=cons1;start='0 11 * * *';duration=300 "
                    "--time-constraint "
                    "name=cons2;start='0 23 * * *';duration=600 "
                    "--query %s "
                    % QUERY))
        alarm = self.details_multiple(result)[0]
        alarm_ptc_id = alarm["alarm_id"]
        self.assertEqual('alarm_ptc', alarm['name'])
        self.assertEqual('80.0', alarm['threshold'])
        self.assertIsNotNone(alarm['time_constraints'])

        # CREATE FAIL MISSING PARAM
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("create "
                                  "--type prometheus "
                                  "--name alarm1 "
                                  "--query %s "
                                  % QUERY))

        # UPDATE
        result = self.aodh(
            'alarm', params=("update %s --severity critical --threshold 90"
                             % ALARM_ID))
        alarm_updated = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_updated["alarm_id"])
        self.assertEqual('critical', alarm_updated['severity'])
        self.assertEqual('90.0', alarm_updated["threshold"])

        # GET
        result = self.aodh(
            'alarm', params="show %s" % ALARM_ID)
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual('alarm_p1', alarm_show['name'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual(QUERY, alarm_show['query'])

        # GET BY NAME
        result = self.aodh(
            'alarm', params="show --name alarm_p1")
        alarm_show = self.details_multiple(result)[0]
        self.assertEqual(ALARM_ID, alarm_show["alarm_id"])
        self.assertEqual('alarm_p1', alarm_show['name'])
        self.assertEqual('90.0', alarm_show['threshold'])
        self.assertEqual('critical', alarm_show['severity'])
        self.assertEqual(QUERY, alarm_show['query'])

        # GET BY NAME AND ID ERROR
        self.assertRaises(exceptions.CommandFailed,
                          self.aodh, 'alarm',
                          params=("show %s --name alarm_p1" %
                                  ALARM_ID))

        # LIST
        result = self.aodh('alarm', params="list --filter all_projects=true")
        self.assertIn(ALARM_ID,
                      [r['alarm_id'] for r in self.parser.listing(result)])
        output_colums = ['alarm_id', 'type', 'name', 'state', 'severity',
                         'enabled']
        for alarm_list in self.parser.listing(result):
            self.assertEqual(sorted(output_colums), sorted(alarm_list.keys()))
            if alarm_list["alarm_id"] == ALARM_ID:
                self.assertEqual('alarm_p1', alarm_list['name'])

        # LIST WITH PAGINATION
        # list with limit
        result = self.aodh('alarm',
                           params="list --filter all_projects=true --limit 1")
        alarm_list = self.parser.listing(result)
        self.assertEqual(1, len(alarm_list))
        # list with sort with key=name dir=asc
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --sort name:asc")
        names = [r['name'] for r in self.parser.listing(result)]
        sorted_name = sorted(names)
        self.assertEqual(sorted_name, names)
        # list with sort with key=name dir=asc and key=alarm_id dir=asc
        result = self.aodh(
            'alarm',
            params=("create --type prometheus "
                    "--name alarm_p2 --threshold 80 "
                    "--query %s"
                    % QUERY))
        created_alarm_id = self.details_multiple(result)[0]['alarm_id']
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --sort name:asc "
                   "--sort alarm_id:asc")
        alarm_list = self.parser.listing(result)
        ids_with_same_name = []
        names = []
        for alarm in alarm_list:
            names.append(['alarm_name'])
            if alarm['name'] == 'alarm_p2':
                ids_with_same_name.append(alarm['alarm_id'])
        sorted_ids = sorted(ids_with_same_name)
        sorted_names = sorted(names)
        self.assertEqual(sorted_names, names)
        self.assertEqual(sorted_ids, ids_with_same_name)
        # List with sort with key=name dir=desc and with the marker
        # verify that we handle non full and empty pages correctly.
        # Verify that marker works as expected

        # We have 3 alarms. List the first 2.
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --filter "
                   "type=prometheus --sort name:asc --limit 2")
        alarm_list = self.parser.listing(result)
        self.assertNotIn('alarm_ptc',
                         [r['name'] for r in alarm_list])
        self.assertEqual(2, len(alarm_list))

        # List the next page. Only the 3rd alarm should be returned
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --filter "
                   "type=prometheus --sort name:asc "
                   "--marker %s --limit 2" % alarm_list[1]['alarm_id'])
        alarm_list = self.parser.listing(result)
        self.assertIn('alarm_ptc',
                      [r['name'] for r in alarm_list])
        self.assertEqual(1, len(alarm_list))

        # List the next page. Empty page should be returned
        result = self.aodh(
            'alarm',
            params="list --filter all_projects=true --filter "
                   "type=prometheus --sort name:asc "
                   "--marker %s --limit 2" % alarm_list[0]['alarm_id'])
        alarm_list = self.parser.listing(result)
        self.assertEqual(0, len(alarm_list))

        # Delete the last created alarm
        self.aodh('alarm', params="delete %s" % created_alarm_id)
        # Delete alarm_ptc
        self.aodh('alarm', params="delete %s" % alarm_ptc_id)

        # DELETE
        result = self.aodh('alarm', params="delete %s" % ALARM_ID)
        self.assertEqual("", result)

        # GET FAIL
        result = self.aodh('alarm', params="show %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        expected = "Alarm %s not found (HTTP 404)" % ALARM_ID
        self.assertFirstLineStartsWith(result.splitlines(), expected)

        # DELETE FAIL
        result = self.aodh('alarm', params="delete %s" % ALARM_ID,
                           fail_ok=True, merge_stderr=True)
        self.assertFirstLineStartsWith(result.splitlines(), expected)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import re
from urllib import parse as urllib_parse

import pyparsing as pp

uninary_operators = ("not", )
binary_operator = (">=", "<=", "!=", ">", "<", "=", "==", "eq", "ne",
                   "lt", "gt", "ge", "le")
multiple_operators = ("and", "or")

operator = pp.Regex("|".join(binary_operator))
null = pp.Regex("None|none|null").setParseAction(pp.replaceWith(None))
boolean = "False|True|false|true"
boolean = pp.Regex(boolean).setParseAction(lambda t: t[0].lower() == "true")
hex_string = lambda n: pp.Word(pp.hexnums, exact=n)  # noqa: E731
uuid = pp.Combine(hex_string(8) + ("-" + hex_string(4)) * 3 +
                  "-" + hex_string(12))
number = r"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?"
number = pp.Regex(number).setParseAction(lambda t: float(t[0]))
identifier = pp.Word(pp.alphas, pp.alphanums + "_")
quoted_string = pp.QuotedString('"') | pp.QuotedString("'")
comparison_term = pp.Forward()
in_list = pp.Group(pp.Suppress('[') +
                   pp.Optional(pp.delimitedList(comparison_term)) +
                   pp.Suppress(']'))("list")
comparison_term << (null | boolean | uuid | identifier | number |
                    quoted_string)
condition = pp.Group(comparison_term + operator + comparison_term)

expr = pp.infixNotation(condition, [
    ("not", 1, pp.opAssoc.RIGHT, ),
    ("and", 2, pp.opAssoc.LEFT, ),
    ("or", 2, pp.opAssoc.LEFT, ),
])

OP_LOOKUP = {'!=': 'ne',
             '>=': 'ge',
             '<=': 'le',
             '>': 'gt',
             '<': 'lt',
             '=': 'eq'}

OP_LOOKUP_KEYS = '|'.join(sorted(OP_LOOKUP.keys(), key=len, reverse=True))
OP_SPLIT_RE = re.compile(r'(%s)' % OP_LOOKUP_KEYS)


def _parsed_query2dict(parsed_query):
    result = None
    while parsed_query:
        part = parsed_query.pop()
        if part in binary_operator:
            result = {part: {parsed_query.pop(): result}}

        elif part in multiple_operators:
            if result.get(part):
                result[part].append(
                    _parsed_query2dict(parsed_query.pop()))
            else:
                result = {part: [result]}

        elif part in uninary_operators:
            result = {part: result}
        elif isinstance(part, pp.ParseResults):
            kind = part.getName()
            if kind == "list":
                res = part.asList()
            else:
                res = _parsed_query2dict(part)
            if result is None:
                result = res
            elif isinstance(result, dict):
                list(result.values())[0].append(res)
        else:
            result = part
    return result


def search_query_builder(query):
    parsed_query = expr.parseString(query)[0]
    return _parsed_query2dict(parsed_query)


def list2cols(cols, objs):
    return cols, [tuple([o[k] for k in cols])
                  for o in objs]


def format_string_list(objs, field):
    objs[field] = ", ".join(objs[field])


def format_dict_list(objs, field):
    objs[field] = "\n".join(
        "- " + ", ".join(f"{k}: {v}"
                         for k, v in elem.items())
        for elem in objs[field])


def format_move_dict_to_root(obj, field):
    for attr in obj[field]:
        obj[f"{field}/{attr}"] = obj[field][attr]
    del obj[field]


def format_archive_policy(ap):
    format_dict_list(ap, "definition")
    format_string_list(ap, "aggregation_methods")


def dict_from_parsed_args(parsed_args, attrs):
    d = {}
    for attr in attrs:
        if attr == "metric":
            if parsed_args.metrics:
                value = parsed_args.metrics[0]
            else:
                value = None
        else:
            value = getattr(parsed_args, attr)
        if value is not None:
            if value == [""]:
                # NOTE(jake): As options like --alarm-actions is an array,
                # their value can be array with empty string if user set option
                # to ''. In this case we set it to None here so that a None
                # value gets sent to API.
                d[attr] = None
            else:
                d[attr] = value
    return d


def dict_to_querystring(objs):
    return "&".join([f"{k}={v}"
                     for k, v in objs.items()
                     if v is not None])


def cli_to_array(cli_query):
    """Convert CLI list of queries to the Python API format.

    This will convert the following:
        "this<=34;that=string::foo"
    to
        "[{field=this,op=le,value=34,type=''},
          {field=that,op=eq,value=foo,type=string}]"

    """

    opts = []
    queries = cli_query.split(';')
    for q in queries:
        try:
            field, q_operator, type_value = OP_SPLIT_RE.split(q, maxsplit=1)
        except ValueError:
            raise ValueError('Invalid or missing operator in query %(q)s,'
                             'the supported operators are: %(k)s' %
                             {'q': q, 'k': OP_LOOKUP.keys()})
        if not field:
            raise ValueError('Missing field in query %s' % q)
        if not type_value:
            raise ValueError('Missing value in query %s' % q)
        opt = dict(field=field, op=OP_LOOKUP[q_operator])

        if '::' not in type_value:
            opt['type'], opt['value'] = '', type_value
        else:
            opt['type'], _, opt['value'] = type_value.partition('::')

        if opt['type'] and opt['type'] not in (
                'string', 'integer', 'float', 'datetime', 'boolean'):
            err = ('Invalid value type %(type)s, the type of value'
                   'should be one of: integer, string, float, datetime,'
                   ' boolean.' % opt)
            raise ValueError(err)
        opts.append(opt)
    return opts


def get_pagination_options(limit=None, marker=None, sorts=None):
    options = []
    if limit:
        options.append("limit=%d" % limit)
    if marker:
        options.append("marker=%s" % urllib_parse.quote(marker))
    for sort in sorts or []:
        options.append("sort=%s" % urllib_parse.quote(sort))
    return "&".join(options)


def get_client(obj):
    if hasattr(obj.app, 'client_manager'):
        # NOTE(liusheng): cliff objects loaded by OSC
        return obj.app.client_manager.alarming
    else:
        # TODO(liusheng): Remove this when OSC is able
        # to install the aodh client binary itself
        return obj.app.client



#
#    Copyright IBM 2016. All rights reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from unittest import mock

import testtools

from aodhclient.v2 import alarm_history


class AlarmHistoryManagerTest(testtools.TestCase):

    def setUp(self):
        super().setUp()
        self.client = mock.Mock()

    @mock.patch.object(alarm_history.AlarmHistoryManager, '_get')
    def test_get(self, mock_ahm):
        ahm = alarm_history.AlarmHistoryManager(self.client)
        ahm.get('01919bbd-8b0e-451c-be28-abe250ae9b1b')
        mock_ahm.assert_called_with(
            'v2/alarms/01919bbd-8b0e-451c-be28-abe250ae9b1b/history')

    @mock.patch.object(alarm_history.AlarmHistoryManager, '_post')
    def test_search(self, mock_ahm):
        ahm = alarm_history.AlarmHistoryManager(self.client)
        q = ('{"and": [{"=": {"type": "gnocchi_resources_threshold"}}, '
             '{"=": {"alarm_id": "87bacbcb-a09c-4cb9-86d0-ad410dd8ad98"}}]}')
        ahm.search(q)
        expected_called_data = (
            '{"filter": "{\\"and\\": ['
            '{\\"=\\": {\\"type\\": \\"gnocchi_resources_threshold\\"}}, '
            '{\\"=\\": {\\"alarm_id\\": '
            '\\"87bacbcb-a09c-4cb9-86d0-ad410dd8ad98\\"}}]}"}')
        mock_ahm.assert_called_with(
            'v2/query/alarms/history',
            data=expected_called_data,
            headers={'Content-Type': 'application/json'})



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslotest import base

from aodhclient import exceptions


class AodhclientExceptionsTest(base.BaseTestCase):
    def test_string_format_base_exception(self):
        # ensure http_status has initial value N/A
        self.assertEqual('Unknown Error (HTTP N/A)',
                         '%s' % exceptions.ClientException())

    def test_no_match_exception_from_response(self):
        resp = mock.MagicMock(status_code=520)
        resp.headers = {
            'Content-Type': 'text/plain',
            'x-openstack-request-id': 'fake-request-id'
        }
        resp.text = 'Of course I still love you'
        e = exceptions.from_response(resp, 'http://no.where:2333/v2/alarms')
        self.assertIsInstance(e, exceptions.ClientException)
        self.assertEqual('Of course I still love you (HTTP 520) '
                         '(Request-ID: fake-request-id)', '%s' % e)
